from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="QuoteHomeConversionFactors")


@dataclasses.dataclass
class QuoteHomeConversionFactors:
    """QuoteHomeConversionFactors represents the factors that can be used used to convert quantities of a Price's
    Instrument's quote currency into the Account's home currency.

        Attributes:
            positive_units (Optional[str]): The factor used to convert a positive amount of the Price's Instrument's
                quote currency into a positive amount of the Account's home currency.  Conversion is performed by multiplying
                the quote units by the conversion factor.
            negative_units (Optional[str]): The factor used to convert a negative amount of the Price's Instrument's
                quote currency into a negative amount of the Account's home currency.  Conversion is performed by multiplying
                the quote units by the conversion factor."""

    positive_units: Optional[str]
    negative_units: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        positive_units = d.pop("positiveUnits", None)
        negative_units = d.pop("negativeUnits", None)
        quote_home_conversion_factors = cls(
            positive_units=positive_units, negative_units=negative_units
        )
        quote_home_conversion_factors.additional_properties = d
        return quote_home_conversion_factors

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .price import Price
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetInstrumentPriceResponse200")


@dataclasses.dataclass
class GetInstrumentPriceResponse200:
    """Attributes:
    price (Optional[Price]): The Price representation"""

    price: Optional["Price"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .price import Price

        d = src_dict.copy()
        _price = d.pop("price", None)
        price: Optional[Price]
        if _price is None:
            price = None
        else:
            price = Price.from_dict(_price)
        get_instrument_price_response_200 = cls(price=price)
        get_instrument_price_response_200.additional_properties = d
        return get_instrument_price_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MT4TransactionHeartbeat")


@dataclasses.dataclass
class MT4TransactionHeartbeat:
    """A TransactionHeartbeat object is injected into the Transaction stream to ensure that the HTTP connection remains
    active.

        Attributes:
            type (Optional[str]): The string "HEARTBEAT"
            time (Optional[str]): The date/time when the TransactionHeartbeat was created."""

    type: Optional[str]
    time: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", None)
        time = d.pop("time", None)
        mt4_transaction_heartbeat = cls(type=type, time=time)
        mt4_transaction_heartbeat.additional_properties = d
        return mt4_transaction_heartbeat

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

TrailingStopLossOrderRejectTransactionTimeInForce = Literal[
    "FOK", "GFD", "GTC", "GTD", "IOC"
]
TRAILING_STOP_LOSS_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES: Set[
    TrailingStopLossOrderRejectTransactionTimeInForce
] = {"FOK", "GFD", "GTC", "GTD", "IOC"}


def check_trailing_stop_loss_order_reject_transaction_time_in_force(
    value: str,
) -> TrailingStopLossOrderRejectTransactionTimeInForce:
    if value in TRAILING_STOP_LOSS_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES:
        return cast(TrailingStopLossOrderRejectTransactionTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRAILING_STOP_LOSS_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TransactionHeartbeat")


@dataclasses.dataclass
class TransactionHeartbeat:
    """A TransactionHeartbeat object is injected into the Transaction stream to ensure that the HTTP connection remains
    active.

        Attributes:
            type (Optional[str]): The string "HEARTBEAT"
            last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account
            time (Optional[str]): The date/time when the TransactionHeartbeat was created."""

    type: Optional[str]
    last_transaction_id: Optional[str]
    time: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", None)
        last_transaction_id = d.pop("lastTransactionID", None)
        time = d.pop("time", None)
        transaction_heartbeat = cls(
            type=type, last_transaction_id=last_transaction_id, time=time
        )
        transaction_heartbeat.additional_properties = d
        return transaction_heartbeat

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="SetTradeDependentOrdersBody")


@dataclasses.dataclass
class SetTradeDependentOrdersBody:
    """Attributes:
    take_profit (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit Order to
        be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Take
        Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
    stop_loss (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to be
        created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop Loss,
        or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
    trailing_stop_loss (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the details of a
        Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is filled that opens
        a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order is modified
        directly through the Trade."""

    take_profit: Optional["TakeProfitDetails"]
    stop_loss: Optional["StopLossDetails"]
    trailing_stop_loss: Optional["TrailingStopLossDetails"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .stop_loss_details import StopLossDetails
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .take_profit_details import TakeProfitDetails

        d = src_dict.copy()
        _take_profit = d.pop("takeProfit", None)
        take_profit: Optional[TakeProfitDetails]
        if _take_profit is None:
            take_profit = None
        else:
            take_profit = TakeProfitDetails.from_dict(_take_profit)
        _stop_loss = d.pop("stopLoss", None)
        stop_loss: Optional[StopLossDetails]
        if _stop_loss is None:
            stop_loss = None
        else:
            stop_loss = StopLossDetails.from_dict(_stop_loss)
        _trailing_stop_loss = d.pop("trailingStopLoss", None)
        trailing_stop_loss: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss is None:
            trailing_stop_loss = None
        else:
            trailing_stop_loss = TrailingStopLossDetails.from_dict(_trailing_stop_loss)
        set_trade_dependent_orders_body = cls(
            take_profit=take_profit,
            stop_loss=stop_loss,
            trailing_stop_loss=trailing_stop_loss,
        )
        set_trade_dependent_orders_body.additional_properties = d
        return set_trade_dependent_orders_body

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

PriceStatus = Literal["invalid", "non-tradeable", "tradeable"]

PRICE_STATUS_VALUES: Set[PriceStatus] = {
    "invalid",
    "non-tradeable",
    "tradeable",
}


def check_price_status(value: str) -> PriceStatus:
    if value in PRICE_STATUS_VALUES:
        return cast(PriceStatus, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {PRICE_STATUS_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .user_info import UserInfo
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetUserInfoResponse200")


@dataclasses.dataclass
class GetUserInfoResponse200:
    """Attributes:
    user_info (Optional[UserInfo]): A representation of user information, as provided to the user themself."""

    user_info: Optional["UserInfo"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .user_info import UserInfo

        d = src_dict.copy()
        _user_info = d.pop("userInfo", None)
        user_info: Optional[UserInfo]
        if _user_info is None:
            user_info = None
        else:
            user_info = UserInfo.from_dict(_user_info)
        get_user_info_response_200 = cls(user_info=user_info)
        get_user_info_response_200.additional_properties = d
        return get_user_info_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

TimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]
TIME_IN_FORCE_VALUES: Set[TimeInForce] = {"FOK", "GFD", "GTC", "GTD", "IOC"}


def check_time_in_force(value: str) -> TimeInForce:
    if value in TIME_IN_FORCE_VALUES:
        return cast(TimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TIME_IN_FORCE_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

InstrumentType = Literal["CFD", "CURRENCY", "METAL"]
INSTRUMENT_TYPE_VALUES: Set[InstrumentType] = {"CFD", "CURRENCY", "METAL"}


def check_instrument_type(value: str) -> InstrumentType:
    if value in INSTRUMENT_TYPE_VALUES:
        return cast(InstrumentType, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {INSTRUMENT_TYPE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.fixed_price_order_transaction_position_fill import (
    check_fixed_price_order_transaction_position_fill,
)
from ..models.fixed_price_order_transaction_position_fill import (
    FixedPriceOrderTransactionPositionFill,
)
from ..models.fixed_price_order_transaction_reason import (
    check_fixed_price_order_transaction_reason,
)
from ..models.fixed_price_order_transaction_reason import (
    FixedPriceOrderTransactionReason,
)
from ..models.fixed_price_order_transaction_type import (
    check_fixed_price_order_transaction_type,
)
from ..models.fixed_price_order_transaction_type import FixedPriceOrderTransactionType
from typing import Union

if TYPE_CHECKING:
    from ..models.take_profit_details import TakeProfitDetails
    from ..models.trailing_stop_loss_details import TrailingStopLossDetails
    from ..models.client_extensions import ClientExtensions
    from ..models.stop_loss_details import StopLossDetails


T = TypeVar("T", bound="FixedPriceOrderTransaction")


@_attrs_define
class FixedPriceOrderTransaction:
    """A FixedPriceOrderTransaction represents the creation of a Fixed Price Order in the user's account. A Fixed Price
    Order is an Order that is filled immediately at a specified price.

        Attributes:
            id (Union[Unset, str]): The Transaction's Identifier.
            time (Union[Unset, str]): The date/time when the Transaction was created.
            user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
            account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
            batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
                batch are applied to the Account simultaneously.
            request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
            type (Union[Unset, FixedPriceOrderTransactionType]): The Type of the Transaction. Always set to
                "FIXED_PRICE_ORDER" in a FixedPriceOrderTransaction.
            instrument (Union[Unset, str]): The Fixed Price Order's Instrument.
            units (Union[Unset, str]): The quantity requested to be filled by the Fixed Price Order. A posititive number of
                units results in a long Order, and a negative number of units results in a short Order.
            price (Union[Unset, str]): The price specified for the Fixed Price Order. This price is the exact price that the
                Fixed Price Order will be filled at.
            position_fill (Union[Unset, FixedPriceOrderTransactionPositionFill]): Specification of how Positions in the
                Account are modified when the Order is filled.
            trade_state (Union[Unset, str]): The state that the trade resulting from the Fixed Price Order should be set to.
            reason (Union[Unset, FixedPriceOrderTransactionReason]): The reason that the Fixed Price Order was created
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            take_profit_on_fill (Union[Unset, TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
                Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
                a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
            stop_loss_on_fill (Union[Unset, StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
                be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
                Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
            trailing_stop_loss_on_fill (Union[Unset, TrailingStopLossDetails]): TrailingStopLossDetails specifies the
                details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
                filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
                is modified directly through the Trade.
            trade_client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, FixedPriceOrderTransactionType] = UNSET
    instrument: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    position_fill: Union[Unset, FixedPriceOrderTransactionPositionFill] = UNSET
    trade_state: Union[Unset, str] = UNSET
    reason: Union[Unset, FixedPriceOrderTransactionReason] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    take_profit_on_fill: Union[Unset, "TakeProfitDetails"] = UNSET
    stop_loss_on_fill: Union[Unset, "StopLossDetails"] = UNSET
    trailing_stop_loss_on_fill: Union[Unset, "TrailingStopLossDetails"] = UNSET
    trade_client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        instrument = self.instrument

        units = self.units

        price = self.price

        position_fill: Union[Unset, str] = UNSET
        if not isinstance(self.position_fill, Unset):
            position_fill = self.position_fill

        trade_state = self.trade_state

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        take_profit_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_on_fill, Unset):
            take_profit_on_fill = self.take_profit_on_fill.to_dict()

        stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_on_fill, Unset):
            stop_loss_on_fill = self.stop_loss_on_fill.to_dict()

        trailing_stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = self.trailing_stop_loss_on_fill.to_dict()

        trade_client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_client_extensions, Unset):
            trade_client_extensions = self.trade_client_extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if units is not UNSET:
            field_dict["units"] = units
        if price is not UNSET:
            field_dict["price"] = price
        if position_fill is not UNSET:
            field_dict["positionFill"] = position_fill
        if trade_state is not UNSET:
            field_dict["tradeState"] = trade_state
        if reason is not UNSET:
            field_dict["reason"] = reason
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if take_profit_on_fill is not UNSET:
            field_dict["takeProfitOnFill"] = take_profit_on_fill
        if stop_loss_on_fill is not UNSET:
            field_dict["stopLossOnFill"] = stop_loss_on_fill
        if trailing_stop_loss_on_fill is not UNSET:
            field_dict["trailingStopLossOnFill"] = trailing_stop_loss_on_fill
        if trade_client_extensions is not UNSET:
            field_dict["tradeClientExtensions"] = trade_client_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.take_profit_details import TakeProfitDetails
        from ..models.trailing_stop_loss_details import TrailingStopLossDetails
        from ..models.client_extensions import ClientExtensions
        from ..models.stop_loss_details import StopLossDetails

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, FixedPriceOrderTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_fixed_price_order_transaction_type(_type)

        instrument = d.pop("instrument", UNSET)

        units = d.pop("units", UNSET)

        price = d.pop("price", UNSET)

        _position_fill = d.pop("positionFill", UNSET)
        position_fill: Union[Unset, FixedPriceOrderTransactionPositionFill]
        if isinstance(_position_fill, Unset):
            position_fill = UNSET
        else:
            position_fill = check_fixed_price_order_transaction_position_fill(
                _position_fill
            )

        trade_state = d.pop("tradeState", UNSET)

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, FixedPriceOrderTransactionReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = check_fixed_price_order_transaction_reason(_reason)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        _take_profit_on_fill = d.pop("takeProfitOnFill", UNSET)
        take_profit_on_fill: Union[Unset, TakeProfitDetails]
        if isinstance(_take_profit_on_fill, Unset):
            take_profit_on_fill = UNSET
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)

        _stop_loss_on_fill = d.pop("stopLossOnFill", UNSET)
        stop_loss_on_fill: Union[Unset, StopLossDetails]
        if isinstance(_stop_loss_on_fill, Unset):
            stop_loss_on_fill = UNSET
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)

        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", UNSET)
        trailing_stop_loss_on_fill: Union[Unset, TrailingStopLossDetails]
        if isinstance(_trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = UNSET
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )

        _trade_client_extensions = d.pop("tradeClientExtensions", UNSET)
        trade_client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_trade_client_extensions, Unset):
            trade_client_extensions = UNSET
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )

        fixed_price_order_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            position_fill=position_fill,
            trade_state=trade_state,
            reason=reason,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
        )

        fixed_price_order_transaction.additional_properties = d
        return fixed_price_order_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

StopOrderRequestPositionFill = Literal[
    "DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"
]
STOP_ORDER_REQUEST_POSITION_FILL_VALUES: Set[StopOrderRequestPositionFill] = {
    "DEFAULT",
    "OPEN_ONLY",
    "REDUCE_FIRST",
    "REDUCE_ONLY",
}


def check_stop_order_request_position_fill(value: str) -> StopOrderRequestPositionFill:
    if value in STOP_ORDER_REQUEST_POSITION_FILL_VALUES:
        return cast(StopOrderRequestPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_ORDER_REQUEST_POSITION_FILL_VALUES!r}"
    )



from typing import Literal, Set, cast

StopOrderRejectTransactionReason = Literal["CLIENT_ORDER", "REPLACEMENT"]

STOP_ORDER_REJECT_TRANSACTION_REASON_VALUES: Set[StopOrderRejectTransactionReason] = {
    "CLIENT_ORDER",
    "REPLACEMENT",
}


def check_stop_order_reject_transaction_reason(
    value: str,
) -> StopOrderRejectTransactionReason:
    if value in STOP_ORDER_REJECT_TRANSACTION_REASON_VALUES:
        return cast(StopOrderRejectTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_ORDER_REJECT_TRANSACTION_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .market_order_reject_transaction import MarketOrderRejectTransaction
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="CloseTradeResponse404")


@dataclasses.dataclass
class CloseTradeResponse404:
    """Attributes:
    order_reject_transaction (Optional[MarketOrderRejectTransaction]): A MarketOrderRejectTransaction represents
        the rejection of the creation of a Market Order.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account. Only
        present if the Account exists.
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request. Only present if the Account exists.
    error_code (Optional[str]): The code of the error that has occurred. This field may not be returned for some
        errors.
    error_message (Optional[str]): The human-readable description of the error that has occurred."""

    order_reject_transaction: Optional["MarketOrderRejectTransaction"]
    last_transaction_id: Optional[str]
    related_transaction_i_ds: Optional[List[str]]
    error_code: Optional[str]
    error_message: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .market_order_reject_transaction import MarketOrderRejectTransaction

        d = src_dict.copy()
        _order_reject_transaction = d.pop("orderRejectTransaction", None)
        order_reject_transaction: Optional[MarketOrderRejectTransaction]
        if _order_reject_transaction is None:
            order_reject_transaction = None
        else:
            order_reject_transaction = MarketOrderRejectTransaction.from_dict(
                _order_reject_transaction
            )
        last_transaction_id = d.pop("lastTransactionID", None)
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        error_code = d.pop("errorCode", None)
        error_message = d.pop("errorMessage", None)
        close_trade_response_404 = cls(
            order_reject_transaction=order_reject_transaction,
            last_transaction_id=last_transaction_id,
            related_transaction_i_ds=related_transaction_i_ds,
            error_code=error_code,
            error_message=error_message,
        )
        close_trade_response_404.additional_properties = d
        return close_trade_response_404

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

FundingReason = Literal[
    "ACCOUNT_TRANSFER",
    "ADJUSTMENT",
    "CLIENT_FUNDING",
    "DIVISION_MIGRATION",
    "SITE_MIGRATION",
]

FUNDING_REASON_VALUES: Set[FundingReason] = {
    "ACCOUNT_TRANSFER",
    "ADJUSTMENT",
    "CLIENT_FUNDING",
    "DIVISION_MIGRATION",
    "SITE_MIGRATION",
}


def check_funding_reason(value: str) -> FundingReason:
    if value in FUNDING_REASON_VALUES:
        return cast(FundingReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {FUNDING_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_cancel_reject_transaction import OrderCancelRejectTransaction
from .stop_loss_order_reject_transaction import StopLossOrderRejectTransaction
from .take_profit_order_reject_transaction import TakeProfitOrderRejectTransaction
from .trailing_stop_loss_order_reject_transaction import (
    TrailingStopLossOrderRejectTransaction,
)
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="SetTradeDependentOrdersResponse400")


@dataclasses.dataclass
class SetTradeDependentOrdersResponse400:
    """Attributes:
    take_profit_order_cancel_reject_transaction (Optional[OrderCancelRejectTransaction]): An
        OrderCancelRejectTransaction represents the rejection of the cancellation of an Order in the client's Account.
    take_profit_order_reject_transaction (Optional[TakeProfitOrderRejectTransaction]): A
        TakeProfitOrderRejectTransaction represents the rejection of the creation of a TakeProfit Order.
    stop_loss_order_cancel_reject_transaction (Optional[OrderCancelRejectTransaction]): An
        OrderCancelRejectTransaction represents the rejection of the cancellation of an Order in the client's Account.
    stop_loss_order_reject_transaction (Optional[StopLossOrderRejectTransaction]): A
        StopLossOrderRejectTransaction represents the rejection of the creation of a StopLoss Order.
    trailing_stop_loss_order_cancel_reject_transaction (Optional[OrderCancelRejectTransaction]): An
        OrderCancelRejectTransaction represents the rejection of the cancellation of an Order in the client's Account.
    trailing_stop_loss_order_reject_transaction (Optional[TrailingStopLossOrderRejectTransaction]): A
        TrailingStopLossOrderRejectTransaction represents the rejection of the creation of a TrailingStopLoss Order.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account.
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request.
    error_code (Optional[str]): The code of the error that has occurred. This field may not be returned for some
        errors.
    error_message (Optional[str]): The human-readable description of the error that has occurred."""

    take_profit_order_cancel_reject_transaction: Optional[
        "OrderCancelRejectTransaction"
    ]
    take_profit_order_reject_transaction: Optional["TakeProfitOrderRejectTransaction"]
    stop_loss_order_cancel_reject_transaction: Optional["OrderCancelRejectTransaction"]
    stop_loss_order_reject_transaction: Optional["StopLossOrderRejectTransaction"]
    trailing_stop_loss_order_cancel_reject_transaction: Optional[
        "OrderCancelRejectTransaction"
    ]
    trailing_stop_loss_order_reject_transaction: Optional[
        "TrailingStopLossOrderRejectTransaction"
    ]
    last_transaction_id: Optional[str]
    related_transaction_i_ds: Optional[List[str]]
    error_code: Optional[str]
    error_message: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order_cancel_reject_transaction import OrderCancelRejectTransaction
        from .take_profit_order_reject_transaction import (
            TakeProfitOrderRejectTransaction,
        )
        from .stop_loss_order_reject_transaction import StopLossOrderRejectTransaction
        from .trailing_stop_loss_order_reject_transaction import (
            TrailingStopLossOrderRejectTransaction,
        )

        d = src_dict.copy()
        _take_profit_order_cancel_reject_transaction = d.pop(
            "takeProfitOrderCancelRejectTransaction", None
        )
        take_profit_order_cancel_reject_transaction: Optional[
            OrderCancelRejectTransaction
        ]
        if _take_profit_order_cancel_reject_transaction is None:
            take_profit_order_cancel_reject_transaction = None
        else:
            take_profit_order_cancel_reject_transaction = (
                OrderCancelRejectTransaction.from_dict(
                    _take_profit_order_cancel_reject_transaction
                )
            )
        _take_profit_order_reject_transaction = d.pop(
            "takeProfitOrderRejectTransaction", None
        )
        take_profit_order_reject_transaction: Optional[TakeProfitOrderRejectTransaction]
        if _take_profit_order_reject_transaction is None:
            take_profit_order_reject_transaction = None
        else:
            take_profit_order_reject_transaction = (
                TakeProfitOrderRejectTransaction.from_dict(
                    _take_profit_order_reject_transaction
                )
            )
        _stop_loss_order_cancel_reject_transaction = d.pop(
            "stopLossOrderCancelRejectTransaction", None
        )
        stop_loss_order_cancel_reject_transaction: Optional[
            OrderCancelRejectTransaction
        ]
        if _stop_loss_order_cancel_reject_transaction is None:
            stop_loss_order_cancel_reject_transaction = None
        else:
            stop_loss_order_cancel_reject_transaction = (
                OrderCancelRejectTransaction.from_dict(
                    _stop_loss_order_cancel_reject_transaction
                )
            )
        _stop_loss_order_reject_transaction = d.pop(
            "stopLossOrderRejectTransaction", None
        )
        stop_loss_order_reject_transaction: Optional[StopLossOrderRejectTransaction]
        if _stop_loss_order_reject_transaction is None:
            stop_loss_order_reject_transaction = None
        else:
            stop_loss_order_reject_transaction = (
                StopLossOrderRejectTransaction.from_dict(
                    _stop_loss_order_reject_transaction
                )
            )
        _trailing_stop_loss_order_cancel_reject_transaction = d.pop(
            "trailingStopLossOrderCancelRejectTransaction", None
        )
        trailing_stop_loss_order_cancel_reject_transaction: Optional[
            OrderCancelRejectTransaction
        ]
        if _trailing_stop_loss_order_cancel_reject_transaction is None:
            trailing_stop_loss_order_cancel_reject_transaction = None
        else:
            trailing_stop_loss_order_cancel_reject_transaction = (
                OrderCancelRejectTransaction.from_dict(
                    _trailing_stop_loss_order_cancel_reject_transaction
                )
            )
        _trailing_stop_loss_order_reject_transaction = d.pop(
            "trailingStopLossOrderRejectTransaction", None
        )
        trailing_stop_loss_order_reject_transaction: Optional[
            TrailingStopLossOrderRejectTransaction
        ]
        if _trailing_stop_loss_order_reject_transaction is None:
            trailing_stop_loss_order_reject_transaction = None
        else:
            trailing_stop_loss_order_reject_transaction = (
                TrailingStopLossOrderRejectTransaction.from_dict(
                    _trailing_stop_loss_order_reject_transaction
                )
            )
        last_transaction_id = d.pop("lastTransactionID", None)
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        error_code = d.pop("errorCode", None)
        error_message = d.pop("errorMessage", None)
        set_trade_dependent_orders_response_400 = cls(
            take_profit_order_cancel_reject_transaction=take_profit_order_cancel_reject_transaction,
            take_profit_order_reject_transaction=take_profit_order_reject_transaction,
            stop_loss_order_cancel_reject_transaction=stop_loss_order_cancel_reject_transaction,
            stop_loss_order_reject_transaction=stop_loss_order_reject_transaction,
            trailing_stop_loss_order_cancel_reject_transaction=trailing_stop_loss_order_cancel_reject_transaction,
            trailing_stop_loss_order_reject_transaction=trailing_stop_loss_order_reject_transaction,
            last_transaction_id=last_transaction_id,
            related_transaction_i_ds=related_transaction_i_ds,
            error_code=error_code,
            error_message=error_message,
        )
        set_trade_dependent_orders_response_400.additional_properties = d
        return set_trade_dependent_orders_response_400

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

LimitOrderTriggerCondition = Literal["ASK", "BID", "DEFAULT", "INVERSE", "MID"]
LIMIT_ORDER_TRIGGER_CONDITION_VALUES: Set[LimitOrderTriggerCondition] = {
    "ASK",
    "BID",
    "DEFAULT",
    "INVERSE",
    "MID",
}


def check_limit_order_trigger_condition(value: str) -> LimitOrderTriggerCondition:
    if value in LIMIT_ORDER_TRIGGER_CONDITION_VALUES:
        return cast(LimitOrderTriggerCondition, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_TRIGGER_CONDITION_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_client_extensions_modify_transaction import (
    OrderClientExtensionsModifyTransaction,
)
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="SetOrderClientExtensionsResponse200")


@dataclasses.dataclass
class SetOrderClientExtensionsResponse200:
    """Attributes:
    order_client_extensions_modify_transaction (Optional[OrderClientExtensionsModifyTransaction]): A
        OrderClientExtensionsModifyTransaction represents the modification of an Order's Client Extensions.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request."""

    order_client_extensions_modify_transaction: Optional[
        "OrderClientExtensionsModifyTransaction"
    ]
    last_transaction_id: Optional[str]
    related_transaction_i_ds: Optional[List[str]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order_client_extensions_modify_transaction import (
            OrderClientExtensionsModifyTransaction,
        )

        d = src_dict.copy()
        _order_client_extensions_modify_transaction = d.pop(
            "orderClientExtensionsModifyTransaction", None
        )
        order_client_extensions_modify_transaction: Optional[
            OrderClientExtensionsModifyTransaction
        ]
        if _order_client_extensions_modify_transaction is None:
            order_client_extensions_modify_transaction = None
        else:
            order_client_extensions_modify_transaction = (
                OrderClientExtensionsModifyTransaction.from_dict(
                    _order_client_extensions_modify_transaction
                )
            )
        last_transaction_id = d.pop("lastTransactionID", None)
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        set_order_client_extensions_response_200 = cls(
            order_client_extensions_modify_transaction=order_client_extensions_modify_transaction,
            last_transaction_id=last_transaction_id,
            related_transaction_i_ds=related_transaction_i_ds,
        )
        set_order_client_extensions_response_200.additional_properties = d
        return set_order_client_extensions_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.order_fill_transaction_reason import check_order_fill_transaction_reason
from ..models.order_fill_transaction_reason import OrderFillTransactionReason
from ..models.order_fill_transaction_type import check_order_fill_transaction_type
from ..models.order_fill_transaction_type import OrderFillTransactionType
from typing import Union

if TYPE_CHECKING:
    from ..models.trade_open import TradeOpen
    from ..models.client_price import ClientPrice
    from ..models.trade_reduce import TradeReduce


T = TypeVar("T", bound="OrderFillTransaction")


@_attrs_define
class OrderFillTransaction:
    """An OrderFillTransaction represents the filling of an Order in the client's Account.

    Attributes:
        id (Union[Unset, str]): The Transaction's Identifier.
        time (Union[Unset, str]): The date/time when the Transaction was created.
        user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
        batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
        type (Union[Unset, OrderFillTransactionType]): The Type of the Transaction. Always set to "ORDER_FILL" for an
            OrderFillTransaction.
        order_id (Union[Unset, str]): The ID of the Order filled.
        client_order_id (Union[Unset, str]): The client Order ID of the Order filled (only provided if the client has
            assigned one).
        instrument (Union[Unset, str]): The name of the filled Order's instrument.
        units (Union[Unset, str]): The number of units filled by the OrderFill.
        gain_quote_home_conversion_factor (Union[Unset, str]): This is the conversion factor in effect for the Account
            at the time of the OrderFill for converting any gains realized in Instrument quote units into units of the
            Account's home currency.
        loss_quote_home_conversion_factor (Union[Unset, str]): This is the conversion factor in effect for the Account
            at the time of the OrderFill for converting any losses realized in Instrument quote units into units of the
            Account's home currency.
        price (Union[Unset, str]): This field is now deprecated and should no longer be used. The individual
            tradesClosed, tradeReduced and tradeOpened fields contain the exact/official price each unit was filled at.
        full_vwap (Union[Unset, str]): The price that all of the units of the OrderFill should have been filled at, in
            the absence of guaranteed price execution. This factors in the Account's current ClientPrice, used liquidity and
            the units of the OrderFill only. If no Trades were closed with their price clamped for guaranteed stop loss
            enforcement, then this value will match the price fields of each Trade opened, closed, and reduced, and they
            will all be the exact same.
        full_price (Union[Unset, ClientPrice]): The specification of an Account-specific Price.
        reason (Union[Unset, OrderFillTransactionReason]): The reason that an Order was filled
        pl (Union[Unset, str]): The profit or loss incurred when the Order was filled.
        financing (Union[Unset, str]): The financing paid or collected when the Order was filled.
        commission (Union[Unset, str]): The commission charged in the Account's home currency as a result of filling the
            Order. The commission is always represented as a positive quantity of the Account's home currency, however it
            reduces the balance in the Account.
        guaranteed_execution_fee (Union[Unset, str]): The total guaranteed execution fees charged for all Trades opened,
            closed or reduced with guaranteed Stop Loss Orders.
        account_balance (Union[Unset, str]): The Account's balance after the Order was filled.
        trade_opened (Union[Unset, TradeOpen]): A TradeOpen object represents a Trade for an instrument that was opened
            in an Account. It is found embedded in Transactions that affect the position of an instrument in the Account,
            specifically the OrderFill Transaction.
        trades_closed (Union[Unset, List['TradeReduce']]): The Trades that were closed when the Order was filled (only
            provided if filling the Order resulted in a closing open Trades).
        trade_reduced (Union[Unset, TradeReduce]): A TradeReduce object represents a Trade for an instrument that was
            reduced (either partially or fully) in an Account. It is found embedded in Transactions that affect the position
            of an instrument in the account, specifically the OrderFill Transaction.
        half_spread_cost (Union[Unset, str]): The half spread cost for the OrderFill, which is the sum of the
            halfSpreadCost values in the tradeOpened, tradesClosed and tradeReduced fields. This can be a positive or
            negative value and is represented in the home currency of the Account.
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, OrderFillTransactionType] = UNSET
    order_id: Union[Unset, str] = UNSET
    client_order_id: Union[Unset, str] = UNSET
    instrument: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    gain_quote_home_conversion_factor: Union[Unset, str] = UNSET
    loss_quote_home_conversion_factor: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    full_vwap: Union[Unset, str] = UNSET
    full_price: Union[Unset, "ClientPrice"] = UNSET
    reason: Union[Unset, OrderFillTransactionReason] = UNSET
    pl: Union[Unset, str] = UNSET
    financing: Union[Unset, str] = UNSET
    commission: Union[Unset, str] = UNSET
    guaranteed_execution_fee: Union[Unset, str] = UNSET
    account_balance: Union[Unset, str] = UNSET
    trade_opened: Union[Unset, "TradeOpen"] = UNSET
    trades_closed: Union[Unset, List["TradeReduce"]] = UNSET
    trade_reduced: Union[Unset, "TradeReduce"] = UNSET
    half_spread_cost: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        order_id = self.order_id

        client_order_id = self.client_order_id

        instrument = self.instrument

        units = self.units

        gain_quote_home_conversion_factor = self.gain_quote_home_conversion_factor

        loss_quote_home_conversion_factor = self.loss_quote_home_conversion_factor

        price = self.price

        full_vwap = self.full_vwap

        full_price: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.full_price, Unset):
            full_price = self.full_price.to_dict()

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason

        pl = self.pl

        financing = self.financing

        commission = self.commission

        guaranteed_execution_fee = self.guaranteed_execution_fee

        account_balance = self.account_balance

        trade_opened: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_opened, Unset):
            trade_opened = self.trade_opened.to_dict()

        trades_closed: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.trades_closed, Unset):
            trades_closed = []
            for trades_closed_item_data in self.trades_closed:
                trades_closed_item = trades_closed_item_data.to_dict()
                trades_closed.append(trades_closed_item)

        trade_reduced: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_reduced, Unset):
            trade_reduced = self.trade_reduced.to_dict()

        half_spread_cost = self.half_spread_cost

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type
        if order_id is not UNSET:
            field_dict["orderID"] = order_id
        if client_order_id is not UNSET:
            field_dict["clientOrderID"] = client_order_id
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if units is not UNSET:
            field_dict["units"] = units
        if gain_quote_home_conversion_factor is not UNSET:
            field_dict["gainQuoteHomeConversionFactor"] = (
                gain_quote_home_conversion_factor
            )
        if loss_quote_home_conversion_factor is not UNSET:
            field_dict["lossQuoteHomeConversionFactor"] = (
                loss_quote_home_conversion_factor
            )
        if price is not UNSET:
            field_dict["price"] = price
        if full_vwap is not UNSET:
            field_dict["fullVWAP"] = full_vwap
        if full_price is not UNSET:
            field_dict["fullPrice"] = full_price
        if reason is not UNSET:
            field_dict["reason"] = reason
        if pl is not UNSET:
            field_dict["pl"] = pl
        if financing is not UNSET:
            field_dict["financing"] = financing
        if commission is not UNSET:
            field_dict["commission"] = commission
        if guaranteed_execution_fee is not UNSET:
            field_dict["guaranteedExecutionFee"] = guaranteed_execution_fee
        if account_balance is not UNSET:
            field_dict["accountBalance"] = account_balance
        if trade_opened is not UNSET:
            field_dict["tradeOpened"] = trade_opened
        if trades_closed is not UNSET:
            field_dict["tradesClosed"] = trades_closed
        if trade_reduced is not UNSET:
            field_dict["tradeReduced"] = trade_reduced
        if half_spread_cost is not UNSET:
            field_dict["halfSpreadCost"] = half_spread_cost

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.trade_open import TradeOpen
        from ..models.client_price import ClientPrice
        from ..models.trade_reduce import TradeReduce

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, OrderFillTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_order_fill_transaction_type(_type)

        order_id = d.pop("orderID", UNSET)

        client_order_id = d.pop("clientOrderID", UNSET)

        instrument = d.pop("instrument", UNSET)

        units = d.pop("units", UNSET)

        gain_quote_home_conversion_factor = d.pop(
            "gainQuoteHomeConversionFactor", UNSET
        )

        loss_quote_home_conversion_factor = d.pop(
            "lossQuoteHomeConversionFactor", UNSET
        )

        price = d.pop("price", UNSET)

        full_vwap = d.pop("fullVWAP", UNSET)

        _full_price = d.pop("fullPrice", UNSET)
        full_price: Union[Unset, ClientPrice]
        if isinstance(_full_price, Unset):
            full_price = UNSET
        else:
            full_price = ClientPrice.from_dict(_full_price)

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, OrderFillTransactionReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = check_order_fill_transaction_reason(_reason)

        pl = d.pop("pl", UNSET)

        financing = d.pop("financing", UNSET)

        commission = d.pop("commission", UNSET)

        guaranteed_execution_fee = d.pop("guaranteedExecutionFee", UNSET)

        account_balance = d.pop("accountBalance", UNSET)

        _trade_opened = d.pop("tradeOpened", UNSET)
        trade_opened: Union[Unset, TradeOpen]
        if isinstance(_trade_opened, Unset):
            trade_opened = UNSET
        else:
            trade_opened = TradeOpen.from_dict(_trade_opened)

        trades_closed = []
        _trades_closed = d.pop("tradesClosed", UNSET)
        for trades_closed_item_data in _trades_closed or []:
            trades_closed_item = TradeReduce.from_dict(trades_closed_item_data)

            trades_closed.append(trades_closed_item)

        _trade_reduced = d.pop("tradeReduced", UNSET)
        trade_reduced: Union[Unset, TradeReduce]
        if isinstance(_trade_reduced, Unset):
            trade_reduced = UNSET
        else:
            trade_reduced = TradeReduce.from_dict(_trade_reduced)

        half_spread_cost = d.pop("halfSpreadCost", UNSET)

        order_fill_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            order_id=order_id,
            client_order_id=client_order_id,
            instrument=instrument,
            units=units,
            gain_quote_home_conversion_factor=gain_quote_home_conversion_factor,
            loss_quote_home_conversion_factor=loss_quote_home_conversion_factor,
            price=price,
            full_vwap=full_vwap,
            full_price=full_price,
            reason=reason,
            pl=pl,
            financing=financing,
            commission=commission,
            guaranteed_execution_fee=guaranteed_execution_fee,
            account_balance=account_balance,
            trade_opened=trade_opened,
            trades_closed=trades_closed,
            trade_reduced=trade_reduced,
            half_spread_cost=half_spread_cost,
        )

        order_fill_transaction.additional_properties = d
        return order_fill_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

LimitOrderRejectTransactionTimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]

LIMIT_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES: Set[
    LimitOrderRejectTransactionTimeInForce
] = {
    "FOK",
    "GFD",
    "GTC",
    "GTD",
    "IOC",
}


def check_limit_order_reject_transaction_time_in_force(
    value: str,
) -> LimitOrderRejectTransactionTimeInForce:
    if value in LIMIT_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES:
        return cast(LimitOrderRejectTransactionTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .calculated_position_state import CalculatedPositionState
from .calculated_trade_state import CalculatedTradeState
from .dynamic_order_state import DynamicOrderState
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="AccountChangesState")


@dataclasses.dataclass
class AccountChangesState:
    """An AccountState Object is used to represent an Account's current price-dependent state. Price-dependent Account
    state is dependent on OANDA's current Prices, and includes things like unrealized PL, NAV and Trailing Stop Loss
    Order state.

        Attributes:
            unrealized_pl (Optional[str]): The total unrealized profit/loss for all Trades currently open in the
                Account.
            nav (Optional[str]): The net asset value of the Account. Equal to Account balance + unrealizedPL.
            margin_used (Optional[str]): Margin currently used for the Account.
            margin_available (Optional[str]): Margin available for Account currency.
            position_value (Optional[str]): The value of the Account's open positions represented in the Account's home
                currency.
            margin_closeout_unrealized_pl (Optional[str]): The Account's margin closeout unrealized PL.
            margin_closeout_nav (Optional[str]): The Account's margin closeout NAV.
            margin_closeout_margin_used (Optional[str]): The Account's margin closeout margin used.
            margin_closeout_percent (Optional[str]): The Account's margin closeout percentage. When this value is 1.0 or
                above the Account is in a margin closeout situation.
            margin_closeout_position_value (Optional[str]): The value of the Account's open positions as used for margin
                closeout calculations represented in the Account's home currency.
            withdrawal_limit (Optional[str]): The current WithdrawalLimit for the account which will be zero or a
                positive value indicating how much can be withdrawn from the account.
            margin_call_margin_used (Optional[str]): The Account's margin call margin used.
            margin_call_percent (Optional[str]): The Account's margin call percentage. When this value is 1.0 or above
                the Account is in a margin call situation.
            orders (Optional[List['DynamicOrderState']]): The price-dependent state of each pending Order in the
                Account.
            trades (Optional[List['CalculatedTradeState']]): The price-dependent state for each open Trade in the
                Account.
            positions (Optional[List['CalculatedPositionState']]): The price-dependent state for each open Position in
                the Account."""

    unrealized_pl: Optional[str]
    nav: Optional[str]
    margin_used: Optional[str]
    margin_available: Optional[str]
    position_value: Optional[str]
    margin_closeout_unrealized_pl: Optional[str]
    margin_closeout_nav: Optional[str]
    margin_closeout_margin_used: Optional[str]
    margin_closeout_percent: Optional[str]
    margin_closeout_position_value: Optional[str]
    withdrawal_limit: Optional[str]
    margin_call_margin_used: Optional[str]
    margin_call_percent: Optional[str]
    orders: Optional[List["DynamicOrderState"]]
    trades: Optional[List["CalculatedTradeState"]]
    positions: Optional[List["CalculatedPositionState"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .calculated_position_state import CalculatedPositionState
        from .dynamic_order_state import DynamicOrderState
        from .calculated_trade_state import CalculatedTradeState

        d = src_dict.copy()
        unrealized_pl = d.pop("unrealizedPL", None)
        nav = d.pop("NAV", None)
        margin_used = d.pop("marginUsed", None)
        margin_available = d.pop("marginAvailable", None)
        position_value = d.pop("positionValue", None)
        margin_closeout_unrealized_pl = d.pop("marginCloseoutUnrealizedPL", None)
        margin_closeout_nav = d.pop("marginCloseoutNAV", None)
        margin_closeout_margin_used = d.pop("marginCloseoutMarginUsed", None)
        margin_closeout_percent = d.pop("marginCloseoutPercent", None)
        margin_closeout_position_value = d.pop("marginCloseoutPositionValue", None)
        withdrawal_limit = d.pop("withdrawalLimit", None)
        margin_call_margin_used = d.pop("marginCallMarginUsed", None)
        margin_call_percent = d.pop("marginCallPercent", None)
        orders = []
        _orders = d.pop("orders", None)
        for orders_item_data in _orders or []:
            orders_item = DynamicOrderState.from_dict(orders_item_data)
            orders.append(orders_item)
        trades = []
        _trades = d.pop("trades", None)
        for trades_item_data in _trades or []:
            trades_item = CalculatedTradeState.from_dict(trades_item_data)
            trades.append(trades_item)
        positions = []
        _positions = d.pop("positions", None)
        for positions_item_data in _positions or []:
            positions_item = CalculatedPositionState.from_dict(positions_item_data)
            positions.append(positions_item)
        account_changes_state = cls(
            unrealized_pl=unrealized_pl,
            nav=nav,
            margin_used=margin_used,
            margin_available=margin_available,
            position_value=position_value,
            margin_closeout_unrealized_pl=margin_closeout_unrealized_pl,
            margin_closeout_nav=margin_closeout_nav,
            margin_closeout_margin_used=margin_closeout_margin_used,
            margin_closeout_percent=margin_closeout_percent,
            margin_closeout_position_value=margin_closeout_position_value,
            withdrawal_limit=withdrawal_limit,
            margin_call_margin_used=margin_call_margin_used,
            margin_call_percent=margin_call_percent,
            orders=orders,
            trades=trades,
            positions=positions,
        )
        account_changes_state.additional_properties = d
        return account_changes_state

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="CalculatedAccountState")


@dataclasses.dataclass
class CalculatedAccountState:
    """The dynamically calculated state of a client's Account.

    Attributes:
        unrealized_pl (Optional[str]): The total unrealized profit/loss for all Trades currently open in the
            Account.
        nav (Optional[str]): The net asset value of the Account. Equal to Account balance + unrealizedPL.
        margin_used (Optional[str]): Margin currently used for the Account.
        margin_available (Optional[str]): Margin available for Account currency.
        position_value (Optional[str]): The value of the Account's open positions represented in the Account's home
            currency.
        margin_closeout_unrealized_pl (Optional[str]): The Account's margin closeout unrealized PL.
        margin_closeout_nav (Optional[str]): The Account's margin closeout NAV.
        margin_closeout_margin_used (Optional[str]): The Account's margin closeout margin used.
        margin_closeout_percent (Optional[str]): The Account's margin closeout percentage. When this value is 1.0 or
            above the Account is in a margin closeout situation.
        margin_closeout_position_value (Optional[str]): The value of the Account's open positions as used for margin
            closeout calculations represented in the Account's home currency.
        withdrawal_limit (Optional[str]): The current WithdrawalLimit for the account which will be zero or a
            positive value indicating how much can be withdrawn from the account.
        margin_call_margin_used (Optional[str]): The Account's margin call margin used.
        margin_call_percent (Optional[str]): The Account's margin call percentage. When this value is 1.0 or above
            the Account is in a margin call situation."""

    unrealized_pl: Optional[str]
    nav: Optional[str]
    margin_used: Optional[str]
    margin_available: Optional[str]
    position_value: Optional[str]
    margin_closeout_unrealized_pl: Optional[str]
    margin_closeout_nav: Optional[str]
    margin_closeout_margin_used: Optional[str]
    margin_closeout_percent: Optional[str]
    margin_closeout_position_value: Optional[str]
    withdrawal_limit: Optional[str]
    margin_call_margin_used: Optional[str]
    margin_call_percent: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        unrealized_pl = d.pop("unrealizedPL", None)
        nav = d.pop("NAV", None)
        margin_used = d.pop("marginUsed", None)
        margin_available = d.pop("marginAvailable", None)
        position_value = d.pop("positionValue", None)
        margin_closeout_unrealized_pl = d.pop("marginCloseoutUnrealizedPL", None)
        margin_closeout_nav = d.pop("marginCloseoutNAV", None)
        margin_closeout_margin_used = d.pop("marginCloseoutMarginUsed", None)
        margin_closeout_percent = d.pop("marginCloseoutPercent", None)
        margin_closeout_position_value = d.pop("marginCloseoutPositionValue", None)
        withdrawal_limit = d.pop("withdrawalLimit", None)
        margin_call_margin_used = d.pop("marginCallMarginUsed", None)
        margin_call_percent = d.pop("marginCallPercent", None)
        calculated_account_state = cls(
            unrealized_pl=unrealized_pl,
            nav=nav,
            margin_used=margin_used,
            margin_available=margin_available,
            position_value=position_value,
            margin_closeout_unrealized_pl=margin_closeout_unrealized_pl,
            margin_closeout_nav=margin_closeout_nav,
            margin_closeout_margin_used=margin_closeout_margin_used,
            margin_closeout_percent=margin_closeout_percent,
            margin_closeout_position_value=margin_closeout_position_value,
            withdrawal_limit=withdrawal_limit,
            margin_call_margin_used=margin_call_margin_used,
            margin_call_percent=margin_call_percent,
        )
        calculated_account_state.additional_properties = d
        return calculated_account_state

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.market_order_margin_closeout_reason import (
    check_market_order_margin_closeout_reason,
)
from ..models.market_order_margin_closeout_reason import MarketOrderMarginCloseoutReason
from typing import Union


T = TypeVar("T", bound="MarketOrderMarginCloseout")


@_attrs_define
class MarketOrderMarginCloseout:
    """Details for the Market Order extensions specific to a Market Order placed that is part of a Market Order Margin
    Closeout in a client's account

        Attributes:
            reason (Union[Unset, MarketOrderMarginCloseoutReason]): The reason the Market Order was created to perform a
                margin closeout
    """

    reason: Union[Unset, MarketOrderMarginCloseoutReason] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if reason is not UNSET:
            field_dict["reason"] = reason

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, MarketOrderMarginCloseoutReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = check_market_order_margin_closeout_reason(_reason)

        market_order_margin_closeout = cls(
            reason=reason,
        )

        market_order_margin_closeout.additional_properties = d
        return market_order_margin_closeout

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="PriceBucket")


@dataclasses.dataclass
class PriceBucket:
    """A Price Bucket represents a price available for an amount of liquidity

    Attributes:
        price (Optional[str]): The Price offered by the PriceBucket
        liquidity (Optional[int]): The amount of liquidity offered by the PriceBucket"""

    price: Optional[str]
    liquidity: Optional[int]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        price = d.pop("price", None)
        liquidity = d.pop("liquidity", None)
        price_bucket = cls(price=price, liquidity=liquidity)
        price_bucket.additional_properties = d
        return price_bucket

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .market_order_margin_closeout_reason import MarketOrderMarginCloseoutReason
from .market_order_margin_closeout_reason import (
    check_market_order_margin_closeout_reason,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MarketOrderMarginCloseout")


@dataclasses.dataclass
class MarketOrderMarginCloseout:
    """Details for the Market Order extensions specific to a Market Order placed that is part of a Market Order Margin
    Closeout in a client's account

        Attributes:
            reason (Optional[MarketOrderMarginCloseoutReason]): The reason the Market Order was created to perform a
                margin closeout"""

    reason: Optional[MarketOrderMarginCloseoutReason]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _reason = d.pop("reason", None)
        reason: Optional[MarketOrderMarginCloseoutReason]
        if _reason is None:
            reason = None
        else:
            reason = check_market_order_margin_closeout_reason(_reason)
        market_order_margin_closeout = cls(reason=reason)
        market_order_margin_closeout.additional_properties = d
        return market_order_margin_closeout

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.take_profit_order_reject_transaction_reason import (
    check_take_profit_order_reject_transaction_reason,
)
from ..models.take_profit_order_reject_transaction_reason import (
    TakeProfitOrderRejectTransactionReason,
)
from ..models.take_profit_order_reject_transaction_reject_reason import (
    check_take_profit_order_reject_transaction_reject_reason,
)
from ..models.take_profit_order_reject_transaction_reject_reason import (
    TakeProfitOrderRejectTransactionRejectReason,
)
from ..models.take_profit_order_reject_transaction_time_in_force import (
    check_take_profit_order_reject_transaction_time_in_force,
)
from ..models.take_profit_order_reject_transaction_time_in_force import (
    TakeProfitOrderRejectTransactionTimeInForce,
)
from ..models.take_profit_order_reject_transaction_trigger_condition import (
    check_take_profit_order_reject_transaction_trigger_condition,
)
from ..models.take_profit_order_reject_transaction_trigger_condition import (
    TakeProfitOrderRejectTransactionTriggerCondition,
)
from ..models.take_profit_order_reject_transaction_type import (
    check_take_profit_order_reject_transaction_type,
)
from ..models.take_profit_order_reject_transaction_type import (
    TakeProfitOrderRejectTransactionType,
)
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="TakeProfitOrderRejectTransaction")


@_attrs_define
class TakeProfitOrderRejectTransaction:
    """A TakeProfitOrderRejectTransaction represents the rejection of the creation of a TakeProfit Order.

    Attributes:
        id (Union[Unset, str]): The Transaction's Identifier.
        time (Union[Unset, str]): The date/time when the Transaction was created.
        user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
        batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
        type (Union[Unset, TakeProfitOrderRejectTransactionType]): The Type of the Transaction. Always set to
            "TAKE_PROFIT_ORDER_REJECT" in a TakeProfitOrderRejectTransaction.
        trade_id (Union[Unset, str]): The ID of the Trade to close when the price threshold is breached.
        client_trade_id (Union[Unset, str]): The client ID of the Trade to be closed when the price threshold is
            breached.
        price (Union[Unset, str]): The price threshold specified for the TakeProfit Order. The associated Trade will be
            closed by a market price that is equal to or better than this threshold.
        time_in_force (Union[Unset, TakeProfitOrderRejectTransactionTimeInForce]): The time-in-force requested for the
            TakeProfit Order. Restricted to "GTC", "GFD" and "GTD" for TakeProfit Orders.
        gtd_time (Union[Unset, str]): The date/time when the TakeProfit Order will be cancelled if its timeInForce is
            "GTD".
        trigger_condition (Union[Unset, TakeProfitOrderRejectTransactionTriggerCondition]): Specification of which price
            component should be used when determining if an Order should be triggered and filled. This allows Orders to be
            triggered based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy)
            price depending on the desired behaviour. Orders are always filled using their default price component.
            This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
            condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
            transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
            is set to the default value when indicating the distance from an Order's trigger price, and will always provide
            the default trigger condition when creating or modifying an Order.
            A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
            value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
            a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
        reason (Union[Unset, TakeProfitOrderRejectTransactionReason]): The reason that the Take Profit Order was
            initiated
        client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        order_fill_transaction_id (Union[Unset, str]): The ID of the OrderFill Transaction that caused this Order to be
            created (only provided if this Order was created automatically when another Order was filled).
        intended_replaces_order_id (Union[Unset, str]): The ID of the Order that this Order was intended to replace
            (only provided if this Order was intended to replace an existing Order).
        reject_reason (Union[Unset, TakeProfitOrderRejectTransactionRejectReason]): The reason that the Reject
            Transaction was created
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, TakeProfitOrderRejectTransactionType] = UNSET
    trade_id: Union[Unset, str] = UNSET
    client_trade_id: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, TakeProfitOrderRejectTransactionTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    trigger_condition: Union[
        Unset, TakeProfitOrderRejectTransactionTriggerCondition
    ] = UNSET
    reason: Union[Unset, TakeProfitOrderRejectTransactionReason] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    order_fill_transaction_id: Union[Unset, str] = UNSET
    intended_replaces_order_id: Union[Unset, str] = UNSET
    reject_reason: Union[Unset, TakeProfitOrderRejectTransactionRejectReason] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        trade_id = self.trade_id

        client_trade_id = self.client_trade_id

        price = self.price

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        trigger_condition: Union[Unset, str] = UNSET
        if not isinstance(self.trigger_condition, Unset):
            trigger_condition = self.trigger_condition

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        order_fill_transaction_id = self.order_fill_transaction_id

        intended_replaces_order_id = self.intended_replaces_order_id

        reject_reason: Union[Unset, str] = UNSET
        if not isinstance(self.reject_reason, Unset):
            reject_reason = self.reject_reason

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type
        if trade_id is not UNSET:
            field_dict["tradeID"] = trade_id
        if client_trade_id is not UNSET:
            field_dict["clientTradeID"] = client_trade_id
        if price is not UNSET:
            field_dict["price"] = price
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if trigger_condition is not UNSET:
            field_dict["triggerCondition"] = trigger_condition
        if reason is not UNSET:
            field_dict["reason"] = reason
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if order_fill_transaction_id is not UNSET:
            field_dict["orderFillTransactionID"] = order_fill_transaction_id
        if intended_replaces_order_id is not UNSET:
            field_dict["intendedReplacesOrderID"] = intended_replaces_order_id
        if reject_reason is not UNSET:
            field_dict["rejectReason"] = reject_reason

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, TakeProfitOrderRejectTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_take_profit_order_reject_transaction_type(_type)

        trade_id = d.pop("tradeID", UNSET)

        client_trade_id = d.pop("clientTradeID", UNSET)

        price = d.pop("price", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, TakeProfitOrderRejectTransactionTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_take_profit_order_reject_transaction_time_in_force(
                _time_in_force
            )

        gtd_time = d.pop("gtdTime", UNSET)

        _trigger_condition = d.pop("triggerCondition", UNSET)
        trigger_condition: Union[
            Unset, TakeProfitOrderRejectTransactionTriggerCondition
        ]
        if isinstance(_trigger_condition, Unset):
            trigger_condition = UNSET
        else:
            trigger_condition = (
                check_take_profit_order_reject_transaction_trigger_condition(
                    _trigger_condition
                )
            )

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, TakeProfitOrderRejectTransactionReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = check_take_profit_order_reject_transaction_reason(_reason)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        order_fill_transaction_id = d.pop("orderFillTransactionID", UNSET)

        intended_replaces_order_id = d.pop("intendedReplacesOrderID", UNSET)

        _reject_reason = d.pop("rejectReason", UNSET)
        reject_reason: Union[Unset, TakeProfitOrderRejectTransactionRejectReason]
        if isinstance(_reject_reason, Unset):
            reject_reason = UNSET
        else:
            reject_reason = check_take_profit_order_reject_transaction_reject_reason(
                _reject_reason
            )

        take_profit_order_reject_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            price=price,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            reason=reason,
            client_extensions=client_extensions,
            order_fill_transaction_id=order_fill_transaction_id,
            intended_replaces_order_id=intended_replaces_order_id,
            reject_reason=reject_reason,
        )

        take_profit_order_reject_transaction.additional_properties = d
        return take_profit_order_reject_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .liquidity_regeneration_schedule_step import LiquidityRegenerationScheduleStep
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="LiquidityRegenerationSchedule")


@dataclasses.dataclass
class LiquidityRegenerationSchedule:
    """A LiquidityRegenerationSchedule indicates how liquidity that is used when filling an Order for an instrument is
    regenerated following the fill.  A liquidity regeneration schedule will be in effect until the timestamp of its
    final step, but may be replaced by a schedule created for an Order of the same instrument that is filled while it is
    still in effect.

        Attributes:
            steps (Optional[List['LiquidityRegenerationScheduleStep']]): The steps in the Liquidity Regeneration
                Schedule"""

    steps: Optional[List["LiquidityRegenerationScheduleStep"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .liquidity_regeneration_schedule_step import (
            LiquidityRegenerationScheduleStep,
        )

        d = src_dict.copy()
        steps = []
        _steps = d.pop("steps", None)
        for steps_item_data in _steps or []:
            steps_item = LiquidityRegenerationScheduleStep.from_dict(steps_item_data)
            steps.append(steps_item)
        liquidity_regeneration_schedule = cls(steps=steps)
        liquidity_regeneration_schedule.additional_properties = d
        return liquidity_regeneration_schedule

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

OrderPositionFill = Literal["DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"]

ORDER_POSITION_FILL_VALUES: Set[OrderPositionFill] = {
    "DEFAULT",
    "OPEN_ONLY",
    "REDUCE_FIRST",
    "REDUCE_ONLY",
}


def check_order_position_fill(value: str) -> OrderPositionFill:
    if value in ORDER_POSITION_FILL_VALUES:
        return cast(OrderPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ORDER_POSITION_FILL_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.trade_summary_state import check_trade_summary_state
from ..models.trade_summary_state import TradeSummaryState
from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="TradeSummary")


@_attrs_define
class TradeSummary:
    """The summary of a Trade within an Account. This representation does not provide the full details of the Trade's
    dependent Orders.

        Attributes:
            id (Union[Unset, str]): The Trade's identifier, unique within the Trade's Account.
            instrument (Union[Unset, str]): The Trade's Instrument.
            price (Union[Unset, str]): The execution price of the Trade.
            open_time (Union[Unset, str]): The date/time when the Trade was opened.
            state (Union[Unset, TradeSummaryState]): The current state of the Trade.
            initial_units (Union[Unset, str]): The initial size of the Trade. Negative values indicate a short Trade, and
                positive values indicate a long Trade.
            initial_margin_required (Union[Unset, str]): The margin required at the time the Trade was created. Note, this
                is the 'pure' margin required, it is not the 'effective' margin used that factors in the trade risk if a GSLO is
                attached to the trade.
            current_units (Union[Unset, str]): The number of units currently open for the Trade. This value is reduced to
                0.0 as the Trade is closed.
            realized_pl (Union[Unset, str]): The total profit/loss realized on the closed portion of the Trade.
            unrealized_pl (Union[Unset, str]): The unrealized profit/loss on the open portion of the Trade.
            margin_used (Union[Unset, str]): Margin currently used by the Trade.
            average_close_price (Union[Unset, str]): The average closing price of the Trade. Only present if the Trade has
                been closed or reduced at least once.
            closing_transaction_i_ds (Union[Unset, List[str]]): The IDs of the Transactions that have closed portions of
                this Trade.
            financing (Union[Unset, str]): The financing paid/collected for this Trade.
            close_time (Union[Unset, str]): The date/time when the Trade was fully closed. Only provided for Trades whose
                state is CLOSED.
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            take_profit_order_id (Union[Unset, str]): ID of the Trade's Take Profit Order, only provided if such an Order
                exists.
            stop_loss_order_id (Union[Unset, str]): ID of the Trade's Stop Loss Order, only provided if such an Order
                exists.
            trailing_stop_loss_order_id (Union[Unset, str]): ID of the Trade's Trailing Stop Loss Order, only provided if
                such an Order exists.
    """

    id: Union[Unset, str] = UNSET
    instrument: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    open_time: Union[Unset, str] = UNSET
    state: Union[Unset, TradeSummaryState] = UNSET
    initial_units: Union[Unset, str] = UNSET
    initial_margin_required: Union[Unset, str] = UNSET
    current_units: Union[Unset, str] = UNSET
    realized_pl: Union[Unset, str] = UNSET
    unrealized_pl: Union[Unset, str] = UNSET
    margin_used: Union[Unset, str] = UNSET
    average_close_price: Union[Unset, str] = UNSET
    closing_transaction_i_ds: Union[Unset, List[str]] = UNSET
    financing: Union[Unset, str] = UNSET
    close_time: Union[Unset, str] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    take_profit_order_id: Union[Unset, str] = UNSET
    stop_loss_order_id: Union[Unset, str] = UNSET
    trailing_stop_loss_order_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        instrument = self.instrument

        price = self.price

        open_time = self.open_time

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        initial_units = self.initial_units

        initial_margin_required = self.initial_margin_required

        current_units = self.current_units

        realized_pl = self.realized_pl

        unrealized_pl = self.unrealized_pl

        margin_used = self.margin_used

        average_close_price = self.average_close_price

        closing_transaction_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.closing_transaction_i_ds, Unset):
            closing_transaction_i_ds = self.closing_transaction_i_ds

        financing = self.financing

        close_time = self.close_time

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        take_profit_order_id = self.take_profit_order_id

        stop_loss_order_id = self.stop_loss_order_id

        trailing_stop_loss_order_id = self.trailing_stop_loss_order_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if price is not UNSET:
            field_dict["price"] = price
        if open_time is not UNSET:
            field_dict["openTime"] = open_time
        if state is not UNSET:
            field_dict["state"] = state
        if initial_units is not UNSET:
            field_dict["initialUnits"] = initial_units
        if initial_margin_required is not UNSET:
            field_dict["initialMarginRequired"] = initial_margin_required
        if current_units is not UNSET:
            field_dict["currentUnits"] = current_units
        if realized_pl is not UNSET:
            field_dict["realizedPL"] = realized_pl
        if unrealized_pl is not UNSET:
            field_dict["unrealizedPL"] = unrealized_pl
        if margin_used is not UNSET:
            field_dict["marginUsed"] = margin_used
        if average_close_price is not UNSET:
            field_dict["averageClosePrice"] = average_close_price
        if closing_transaction_i_ds is not UNSET:
            field_dict["closingTransactionIDs"] = closing_transaction_i_ds
        if financing is not UNSET:
            field_dict["financing"] = financing
        if close_time is not UNSET:
            field_dict["closeTime"] = close_time
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if take_profit_order_id is not UNSET:
            field_dict["takeProfitOrderID"] = take_profit_order_id
        if stop_loss_order_id is not UNSET:
            field_dict["stopLossOrderID"] = stop_loss_order_id
        if trailing_stop_loss_order_id is not UNSET:
            field_dict["trailingStopLossOrderID"] = trailing_stop_loss_order_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        instrument = d.pop("instrument", UNSET)

        price = d.pop("price", UNSET)

        open_time = d.pop("openTime", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, TradeSummaryState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_trade_summary_state(_state)

        initial_units = d.pop("initialUnits", UNSET)

        initial_margin_required = d.pop("initialMarginRequired", UNSET)

        current_units = d.pop("currentUnits", UNSET)

        realized_pl = d.pop("realizedPL", UNSET)

        unrealized_pl = d.pop("unrealizedPL", UNSET)

        margin_used = d.pop("marginUsed", UNSET)

        average_close_price = d.pop("averageClosePrice", UNSET)

        closing_transaction_i_ds = cast(
            List[str], d.pop("closingTransactionIDs", UNSET)
        )

        financing = d.pop("financing", UNSET)

        close_time = d.pop("closeTime", UNSET)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        take_profit_order_id = d.pop("takeProfitOrderID", UNSET)

        stop_loss_order_id = d.pop("stopLossOrderID", UNSET)

        trailing_stop_loss_order_id = d.pop("trailingStopLossOrderID", UNSET)

        trade_summary = cls(
            id=id,
            instrument=instrument,
            price=price,
            open_time=open_time,
            state=state,
            initial_units=initial_units,
            initial_margin_required=initial_margin_required,
            current_units=current_units,
            realized_pl=realized_pl,
            unrealized_pl=unrealized_pl,
            margin_used=margin_used,
            average_close_price=average_close_price,
            closing_transaction_i_ds=closing_transaction_i_ds,
            financing=financing,
            close_time=close_time,
            client_extensions=client_extensions,
            take_profit_order_id=take_profit_order_id,
            stop_loss_order_id=stop_loss_order_id,
            trailing_stop_loss_order_id=trailing_stop_loss_order_id,
        )

        trade_summary.additional_properties = d
        return trade_summary

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="InstrumentCommission")


@dataclasses.dataclass
class InstrumentCommission:
    """An InstrumentCommission represents an instrument-specific commission

    Attributes:
        commission (Optional[str]): The commission amount (in the Account's home currency) charged per unitsTraded
            of the instrument
        units_traded (Optional[str]): The number of units traded that the commission amount is based on.
        minimum_commission (Optional[str]): The minimum commission amount (in the Account's home currency) that is
            charged when an Order is filled for this instrument."""

    commission: Optional[str]
    units_traded: Optional[str]
    minimum_commission: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        commission = d.pop("commission", None)
        units_traded = d.pop("unitsTraded", None)
        minimum_commission = d.pop("minimumCommission", None)
        instrument_commission = cls(
            commission=commission,
            units_traded=units_traded,
            minimum_commission=minimum_commission,
        )
        instrument_commission.additional_properties = d
        return instrument_commission

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_book import OrderBook
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetInstrumentsInstrumentOrderBookResponse200")


@dataclasses.dataclass
class GetInstrumentsInstrumentOrderBookResponse200:
    """Attributes:
    order_book (Optional[OrderBook]): The representation of an instrument's order book at a point in time"""

    order_book: Optional["OrderBook"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order_book import OrderBook

        d = src_dict.copy()
        _order_book = d.pop("orderBook", None)
        order_book: Optional[OrderBook]
        if _order_book is None:
            order_book = None
        else:
            order_book = OrderBook.from_dict(_order_book)
        get_instruments_instrument_order_book_response_200 = cls(order_book=order_book)
        get_instruments_instrument_order_book_response_200.additional_properties = d
        return get_instruments_instrument_order_book_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .take_profit_order_transaction_reason import TakeProfitOrderTransactionReason
from .take_profit_order_transaction_reason import (
    check_take_profit_order_transaction_reason,
)
from .take_profit_order_transaction_time_in_force import (
    TakeProfitOrderTransactionTimeInForce,
)
from .take_profit_order_transaction_time_in_force import (
    check_take_profit_order_transaction_time_in_force,
)
from .take_profit_order_transaction_trigger_condition import (
    TakeProfitOrderTransactionTriggerCondition,
)
from .take_profit_order_transaction_trigger_condition import (
    check_take_profit_order_transaction_trigger_condition,
)
from .take_profit_order_transaction_type import TakeProfitOrderTransactionType
from .take_profit_order_transaction_type import check_take_profit_order_transaction_type
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TakeProfitOrderTransaction")


@dataclasses.dataclass
class TakeProfitOrderTransaction:
    """A TakeProfitOrderTransaction represents the creation of a TakeProfit Order in the user's Account.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[TakeProfitOrderTransactionType]): The Type of the Transaction. Always set to
            "TAKE_PROFIT_ORDER" in a TakeProfitOrderTransaction.
        trade_id (Optional[str]): The ID of the Trade to close when the price threshold is breached.
        client_trade_id (Optional[str]): The client ID of the Trade to be closed when the price threshold is
            breached.
        price (Optional[str]): The price threshold specified for the TakeProfit Order. The associated Trade will be
            closed by a market price that is equal to or better than this threshold.
        time_in_force (Optional[TakeProfitOrderTransactionTimeInForce]): The time-in-force requested for the
            TakeProfit Order. Restricted to "GTC", "GFD" and "GTD" for TakeProfit Orders.
        gtd_time (Optional[str]): The date/time when the TakeProfit Order will be cancelled if its timeInForce is
            "GTD".
        trigger_condition (Optional[TakeProfitOrderTransactionTriggerCondition]): Specification of which price
            component should be used when determining if an Order should be triggered and filled. This allows Orders to be
            triggered based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy)
            price depending on the desired behaviour. Orders are always filled using their default price component.
            This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
            condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
            transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
            is set to the default value when indicating the distance from an Order's trigger price, and will always provide
            the default trigger condition when creating or modifying an Order.
            A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
            value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
            a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
        reason (Optional[TakeProfitOrderTransactionReason]): The reason that the Take Profit Order was initiated
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        order_fill_transaction_id (Optional[str]): The ID of the OrderFill Transaction that caused this Order to be
            created (only provided if this Order was created automatically when another Order was filled).
        replaces_order_id (Optional[str]): The ID of the Order that this Order replaces (only provided if this Order
            replaces an existing Order).
        cancelling_transaction_id (Optional[str]): The ID of the Transaction that cancels the replaced Order (only
            provided if this Order replaces an existing Order)."""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[TakeProfitOrderTransactionType]
    trade_id: Optional[str]
    client_trade_id: Optional[str]
    price: Optional[str]
    time_in_force: Optional[TakeProfitOrderTransactionTimeInForce]
    gtd_time: Optional[str]
    trigger_condition: Optional[TakeProfitOrderTransactionTriggerCondition]
    reason: Optional[TakeProfitOrderTransactionReason]
    client_extensions: Optional["ClientExtensions"]
    order_fill_transaction_id: Optional[str]
    replaces_order_id: Optional[str]
    cancelling_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[TakeProfitOrderTransactionType]
        if _type is None:
            type = None
        else:
            type = check_take_profit_order_transaction_type(_type)
        trade_id = d.pop("tradeID", None)
        client_trade_id = d.pop("clientTradeID", None)
        price = d.pop("price", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[TakeProfitOrderTransactionTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_take_profit_order_transaction_time_in_force(
                _time_in_force
            )
        gtd_time = d.pop("gtdTime", None)
        _trigger_condition = d.pop("triggerCondition", None)
        trigger_condition: Optional[TakeProfitOrderTransactionTriggerCondition]
        if _trigger_condition is None:
            trigger_condition = None
        else:
            trigger_condition = check_take_profit_order_transaction_trigger_condition(
                _trigger_condition
            )
        _reason = d.pop("reason", None)
        reason: Optional[TakeProfitOrderTransactionReason]
        if _reason is None:
            reason = None
        else:
            reason = check_take_profit_order_transaction_reason(_reason)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        order_fill_transaction_id = d.pop("orderFillTransactionID", None)
        replaces_order_id = d.pop("replacesOrderID", None)
        cancelling_transaction_id = d.pop("cancellingTransactionID", None)
        take_profit_order_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            price=price,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            reason=reason,
            client_extensions=client_extensions,
            order_fill_transaction_id=order_fill_transaction_id,
            replaces_order_id=replaces_order_id,
            cancelling_transaction_id=cancelling_transaction_id,
        )
        take_profit_order_transaction.additional_properties = d
        return take_profit_order_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_configure_reject_transaction import ClientConfigureRejectTransaction
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="ConfigureAccountResponse400")


@dataclasses.dataclass
class ConfigureAccountResponse400:
    """Attributes:
    client_configure_reject_transaction (Optional[ClientConfigureRejectTransaction]): A
        ClientConfigureRejectTransaction represents the reject of configuration of an Account by a client.
    last_transaction_id (Optional[str]): The ID of the last Transaction created for the Account.
    error_code (Optional[str]): The code of the error that has occurred. This field may not be returned for some
        errors.
    error_message (Optional[str]): The human-readable description of the error that has occurred."""

    client_configure_reject_transaction: Optional["ClientConfigureRejectTransaction"]
    last_transaction_id: Optional[str]
    error_code: Optional[str]
    error_message: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_configure_reject_transaction import (
            ClientConfigureRejectTransaction,
        )

        d = src_dict.copy()
        _client_configure_reject_transaction = d.pop(
            "clientConfigureRejectTransaction", None
        )
        client_configure_reject_transaction: Optional[ClientConfigureRejectTransaction]
        if _client_configure_reject_transaction is None:
            client_configure_reject_transaction = None
        else:
            client_configure_reject_transaction = (
                ClientConfigureRejectTransaction.from_dict(
                    _client_configure_reject_transaction
                )
            )
        last_transaction_id = d.pop("lastTransactionID", None)
        error_code = d.pop("errorCode", None)
        error_message = d.pop("errorMessage", None)
        configure_account_response_400 = cls(
            client_configure_reject_transaction=client_configure_reject_transaction,
            last_transaction_id=last_transaction_id,
            error_code=error_code,
            error_message=error_message,
        )
        configure_account_response_400.additional_properties = d
        return configure_account_response_400

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="ClientExtensions")


@_attrs_define
class ClientExtensions:
    """A ClientExtensions object allows a client to attach a clientID, tag and comment to Orders and Trades in their
    Account.  Do not set, modify, or delete this field if your account is associated with MT4.

        Attributes:
            id (Union[Unset, str]): The Client ID of the Order/Trade
            tag (Union[Unset, str]): A tag associated with the Order/Trade
            comment (Union[Unset, str]): A comment associated with the Order/Trade
    """

    id: Union[Unset, str] = UNSET
    tag: Union[Unset, str] = UNSET
    comment: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        tag = self.tag

        comment = self.comment

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if tag is not UNSET:
            field_dict["tag"] = tag
        if comment is not UNSET:
            field_dict["comment"] = comment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        tag = d.pop("tag", UNSET)

        comment = d.pop("comment", UNSET)

        client_extensions = cls(
            id=id,
            tag=tag,
            comment=comment,
        )

        client_extensions.additional_properties = d
        return client_extensions

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

TakeProfitOrderRequestTimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]
TAKE_PROFIT_ORDER_REQUEST_TIME_IN_FORCE_VALUES: Set[
    TakeProfitOrderRequestTimeInForce
] = {"FOK", "GFD", "GTC", "GTD", "IOC"}


def check_take_profit_order_request_time_in_force(
    value: str,
) -> TakeProfitOrderRequestTimeInForce:
    if value in TAKE_PROFIT_ORDER_REQUEST_TIME_IN_FORCE_VALUES:
        return cast(TakeProfitOrderRequestTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TAKE_PROFIT_ORDER_REQUEST_TIME_IN_FORCE_VALUES!r}"
    )



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.stream_pricing_response_200 import StreamPricingResponse200
from ...types import Unset


def _get_kwargs(
    account_id: str,
    *,
    instruments: List[str],
    snapshot: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["Authorization"] = authorization

    if not isinstance(accept_datetime_format, Unset):
        headers["Accept-Datetime-Format"] = accept_datetime_format

    params: Dict[str, Any] = {}

    json_instruments = instruments

    params["instruments"] = json_instruments

    params["snapshot"] = snapshot

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/accounts/{account_id}/pricing/stream".format(
            account_id=account_id,
        ),
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, StreamPricingResponse200]]:
    if response.status_code == 200:
        response_200 = StreamPricingResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 405:
        response_405 = cast(Any, None)
        return response_405
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, StreamPricingResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    snapshot: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, StreamPricingResponse200]]:
    """Price Stream

     Get a stream of Account Prices starting from when the request is made.
    This pricing stream does not include every single price created for the Account, but instead will
    provide at most 4 prices per second (every 250 milliseconds) for each instrument being requested.
    If more than one price is created for an instrument during the 250 millisecond window, only the
    price in effect at the end of the window is sent. This means that during periods of rapid price
    movement, subscribers to this stream will not be sent every price.
    Pricing windows for different connections to the price stream are not all aligned in the same way
    (i.e. they are not all aligned to the top of the second). This means that during periods of rapid
    price movement, different subscribers may observe different prices depending on their alignment.

    Args:
        account_id (str):
        instruments (List[str]):
        snapshot (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, StreamPricingResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        instruments=instruments,
        snapshot=snapshot,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    snapshot: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, StreamPricingResponse200]]:
    """Price Stream

     Get a stream of Account Prices starting from when the request is made.
    This pricing stream does not include every single price created for the Account, but instead will
    provide at most 4 prices per second (every 250 milliseconds) for each instrument being requested.
    If more than one price is created for an instrument during the 250 millisecond window, only the
    price in effect at the end of the window is sent. This means that during periods of rapid price
    movement, subscribers to this stream will not be sent every price.
    Pricing windows for different connections to the price stream are not all aligned in the same way
    (i.e. they are not all aligned to the top of the second). This means that during periods of rapid
    price movement, different subscribers may observe different prices depending on their alignment.

    Args:
        account_id (str):
        instruments (List[str]):
        snapshot (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, StreamPricingResponse200]
    """

    return sync_detailed(
        account_id=account_id,
        client=client,
        instruments=instruments,
        snapshot=snapshot,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    ).parsed


async def asyncio_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    snapshot: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, StreamPricingResponse200]]:
    """Price Stream

     Get a stream of Account Prices starting from when the request is made.
    This pricing stream does not include every single price created for the Account, but instead will
    provide at most 4 prices per second (every 250 milliseconds) for each instrument being requested.
    If more than one price is created for an instrument during the 250 millisecond window, only the
    price in effect at the end of the window is sent. This means that during periods of rapid price
    movement, subscribers to this stream will not be sent every price.
    Pricing windows for different connections to the price stream are not all aligned in the same way
    (i.e. they are not all aligned to the top of the second). This means that during periods of rapid
    price movement, different subscribers may observe different prices depending on their alignment.

    Args:
        account_id (str):
        instruments (List[str]):
        snapshot (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, StreamPricingResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        instruments=instruments,
        snapshot=snapshot,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: List[str],
    snapshot: Union[Unset, bool] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, StreamPricingResponse200]]:
    """Price Stream

     Get a stream of Account Prices starting from when the request is made.
    This pricing stream does not include every single price created for the Account, but instead will
    provide at most 4 prices per second (every 250 milliseconds) for each instrument being requested.
    If more than one price is created for an instrument during the 250 millisecond window, only the
    price in effect at the end of the window is sent. This means that during periods of rapid price
    movement, subscribers to this stream will not be sent every price.
    Pricing windows for different connections to the price stream are not all aligned in the same way
    (i.e. they are not all aligned to the top of the second). This means that during periods of rapid
    price movement, different subscribers may observe different prices depending on their alignment.

    Args:
        account_id (str):
        instruments (List[str]):
        snapshot (Union[Unset, bool]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, StreamPricingResponse200]
    """

    return (
        await asyncio_detailed(
            account_id=account_id,
            client=client,
            instruments=instruments,
            snapshot=snapshot,
            authorization=authorization,
            accept_datetime_format=accept_datetime_format,
        )
    ).parsed



from __future__ import annotations
from typing import Literal, Set, cast

MarketIfTouchedOrderTransactionPositionFill = Literal[
    "DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"
]
MARKET_IF_TOUCHED_ORDER_TRANSACTION_POSITION_FILL_VALUES: Set[
    MarketIfTouchedOrderTransactionPositionFill
] = {"DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"}


def check_market_if_touched_order_transaction_position_fill(
    value: str,
) -> MarketIfTouchedOrderTransactionPositionFill:
    if value in MARKET_IF_TOUCHED_ORDER_TRANSACTION_POSITION_FILL_VALUES:
        return cast(MarketIfTouchedOrderTransactionPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_IF_TOUCHED_ORDER_TRANSACTION_POSITION_FILL_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order import Order
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="ListPendingOrdersResponse200")


@dataclasses.dataclass
class ListPendingOrdersResponse200:
    """Attributes:
    orders (Optional[List['Order']]): The list of pending Order details
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    orders: Optional[List["Order"]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order import Order

        d = src_dict.copy()
        orders = []
        _orders = d.pop("orders", None)
        for orders_item_data in _orders or []:
            orders_item = Order.from_dict(orders_item_data)
            orders.append(orders_item)
        last_transaction_id = d.pop("lastTransactionID", None)
        list_pending_orders_response_200 = cls(
            orders=orders, last_transaction_id=last_transaction_id
        )
        list_pending_orders_response_200.additional_properties = d
        return list_pending_orders_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .list_transactions_response_200_type_item import (
    ListTransactionsResponse200TypeItem,
)
from .list_transactions_response_200_type_item import (
    check_list_transactions_response_200_type_item,
)
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="ListTransactionsResponse200")


@dataclasses.dataclass
class ListTransactionsResponse200:
    """Attributes:
    from_ (Optional[str]): The starting time provided in the request.
    to (Optional[str]): The ending time provided in the request.
    page_size (Optional[int]): The pageSize provided in the request
    type (Optional[List[ListTransactionsResponse200TypeItem]]): The Transaction-type filter provided in the
        request
    count (Optional[int]): The number of Transactions that are contained in the pages returned
    pages (Optional[List[str]]): The list of URLs that represent idrange queries providing the data for each
        page in the query results
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    from_: Optional[str]
    to: Optional[str]
    page_size: Optional[int]
    type: Optional[List[ListTransactionsResponse200TypeItem]]
    count: Optional[int]
    pages: Optional[List[str]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        from_ = d.pop("from", None)
        to = d.pop("to", None)
        page_size = d.pop("pageSize", None)
        type = []
        _type = d.pop("type", None)
        for type_item_data in _type or []:
            type_item = check_list_transactions_response_200_type_item(type_item_data)
            type.append(type_item)
        count = d.pop("count", None)
        pages = cast(List[str], d.pop("pages", None))
        last_transaction_id = d.pop("lastTransactionID", None)
        list_transactions_response_200 = cls(
            from_=from_,
            to=to,
            page_size=page_size,
            type=type,
            count=count,
            pages=pages,
            last_transaction_id=last_transaction_id,
        )
        list_transactions_response_200.additional_properties = d
        return list_transactions_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .stop_loss_details_time_in_force import StopLossDetailsTimeInForce
from .stop_loss_details_time_in_force import check_stop_loss_details_time_in_force
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="StopLossDetails")


@dataclasses.dataclass
class StopLossDetails:
    """StopLossDetails specifies the details of a Stop Loss Order to be created on behalf of a client. This may happen when
    an Order is filled that opens a Trade requiring a Stop Loss, or when a Trade's dependent Stop Loss Order is modified
    directly through the Trade.

        Attributes:
            price (Optional[str]): The price that the Stop Loss Order will be triggered at. Only one of the price and
                distance fields may be specified.
            distance (Optional[str]): Specifies the distance (in price units) from the Trade's open price to use as the
                Stop Loss Order price. Only one of the distance and price fields may be specified.
            time_in_force (Optional[StopLossDetailsTimeInForce]): The time in force for the created Stop Loss Order.
                This may only be GTC, GTD or GFD.
            gtd_time (Optional[str]): The date when the Stop Loss Order will be cancelled on if timeInForce is GTD.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            guaranteed (Optional[bool]): Flag indicating that the price for the Stop Loss Order is guaranteed. The
                default value depends on the GuaranteedStopLossOrderMode of the account, if it is REQUIRED, the default will be
                true, for DISABLED or ENABLED the default is false."""

    price: Optional[str]
    distance: Optional[str]
    time_in_force: Optional[StopLossDetailsTimeInForce]
    gtd_time: Optional[str]
    client_extensions: Optional["ClientExtensions"]
    guaranteed: Optional[bool]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        price = d.pop("price", None)
        distance = d.pop("distance", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[StopLossDetailsTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_stop_loss_details_time_in_force(_time_in_force)
        gtd_time = d.pop("gtdTime", None)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        guaranteed = d.pop("guaranteed", None)
        stop_loss_details = cls(
            price=price,
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            client_extensions=client_extensions,
            guaranteed=guaranteed,
        )
        stop_loss_details.additional_properties = d
        return stop_loss_details

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

LimitOrderState = Literal["CANCELLED", "FILLED", "PENDING", "TRIGGERED"]
LIMIT_ORDER_STATE_VALUES: Set[LimitOrderState] = {
    "CANCELLED",
    "FILLED",
    "PENDING",
    "TRIGGERED",
}


def check_limit_order_state(value: str) -> LimitOrderState:
    if value in LIMIT_ORDER_STATE_VALUES:
        return cast(LimitOrderState, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_STATE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .order_client_extensions_modify_transaction_type import (
    OrderClientExtensionsModifyTransactionType,
)
from .order_client_extensions_modify_transaction_type import (
    check_order_client_extensions_modify_transaction_type,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="OrderClientExtensionsModifyTransaction")


@dataclasses.dataclass
class OrderClientExtensionsModifyTransaction:
    """A OrderClientExtensionsModifyTransaction represents the modification of an Order's Client Extensions.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[OrderClientExtensionsModifyTransactionType]): The Type of the Transaction. Always set to
            "ORDER_CLIENT_EXTENSIONS_MODIFY" for a OrderClienteExtensionsModifyTransaction.
        order_id (Optional[str]): The ID of the Order who's client extensions are to be modified.
        client_order_id (Optional[str]): The original Client ID of the Order who's client extensions are to be
            modified.
        client_extensions_modify (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        trade_client_extensions_modify (Optional[ClientExtensions]): A ClientExtensions object allows a client to
            attach a clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this
            field if your account is associated with MT4."""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[OrderClientExtensionsModifyTransactionType]
    order_id: Optional[str]
    client_order_id: Optional[str]
    client_extensions_modify: Optional["ClientExtensions"]
    trade_client_extensions_modify: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[OrderClientExtensionsModifyTransactionType]
        if _type is None:
            type = None
        else:
            type = check_order_client_extensions_modify_transaction_type(_type)
        order_id = d.pop("orderID", None)
        client_order_id = d.pop("clientOrderID", None)
        _client_extensions_modify = d.pop("clientExtensionsModify", None)
        client_extensions_modify: Optional[ClientExtensions]
        if _client_extensions_modify is None:
            client_extensions_modify = None
        else:
            client_extensions_modify = ClientExtensions.from_dict(
                _client_extensions_modify
            )
        _trade_client_extensions_modify = d.pop("tradeClientExtensionsModify", None)
        trade_client_extensions_modify: Optional[ClientExtensions]
        if _trade_client_extensions_modify is None:
            trade_client_extensions_modify = None
        else:
            trade_client_extensions_modify = ClientExtensions.from_dict(
                _trade_client_extensions_modify
            )
        order_client_extensions_modify_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            order_id=order_id,
            client_order_id=client_order_id,
            client_extensions_modify=client_extensions_modify,
            trade_client_extensions_modify=trade_client_extensions_modify,
        )
        order_client_extensions_modify_transaction.additional_properties = d
        return order_client_extensions_modify_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="UnitsAvailableDetails")


@dataclasses.dataclass
class UnitsAvailableDetails:
    """Representation of many units of an Instrument are available to be traded for both long and short Orders.

    Attributes:
        long (Optional[str]): The units available for long Orders.
        short (Optional[str]): The units available for short Orders."""

    long: Optional[str]
    short: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        long = d.pop("long", None)
        short = d.pop("short", None)
        units_available_details = cls(long=long, short=short)
        units_available_details.additional_properties = d
        return units_available_details

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="ClosePositionBody")


@dataclasses.dataclass
class ClosePositionBody:
    """Attributes:
    long_units (Optional[str]): Indication of how much of the long Position to closeout. Either the string
        "ALL", the string "NONE", or a DecimalNumber representing how many units of the long position to close using a
        PositionCloseout MarketOrder. The units specified must always be positive.
    long_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
        clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
        your account is associated with MT4.
    short_units (Optional[str]): Indication of how much of the short Position to closeout. Either the string
        "ALL", the string "NONE", or a DecimalNumber representing how many units of the short position to close using a
        PositionCloseout MarketOrder. The units specified must always be positive.
    short_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
        clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
        your account is associated with MT4."""

    long_units: Optional[str]
    long_client_extensions: Optional["ClientExtensions"]
    short_units: Optional[str]
    short_client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        long_units = d.pop("longUnits", None)
        _long_client_extensions = d.pop("longClientExtensions", None)
        long_client_extensions: Optional[ClientExtensions]
        if _long_client_extensions is None:
            long_client_extensions = None
        else:
            long_client_extensions = ClientExtensions.from_dict(_long_client_extensions)
        short_units = d.pop("shortUnits", None)
        _short_client_extensions = d.pop("shortClientExtensions", None)
        short_client_extensions: Optional[ClientExtensions]
        if _short_client_extensions is None:
            short_client_extensions = None
        else:
            short_client_extensions = ClientExtensions.from_dict(
                _short_client_extensions
            )
        close_position_body = cls(
            long_units=long_units,
            long_client_extensions=long_client_extensions,
            short_units=short_units,
            short_client_extensions=short_client_extensions,
        )
        close_position_body.additional_properties = d
        return close_position_body

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_cancel_transaction import OrderCancelTransaction
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="CancelOrderResponse200")


@dataclasses.dataclass
class CancelOrderResponse200:
    """Attributes:
    order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction represents the
        cancellation of an Order in the client's Account.
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    order_cancel_transaction: Optional["OrderCancelTransaction"]
    related_transaction_i_ds: Optional[List[str]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order_cancel_transaction import OrderCancelTransaction

        d = src_dict.copy()
        _order_cancel_transaction = d.pop("orderCancelTransaction", None)
        order_cancel_transaction: Optional[OrderCancelTransaction]
        if _order_cancel_transaction is None:
            order_cancel_transaction = None
        else:
            order_cancel_transaction = OrderCancelTransaction.from_dict(
                _order_cancel_transaction
            )
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        last_transaction_id = d.pop("lastTransactionID", None)
        cancel_order_response_200 = cls(
            order_cancel_transaction=order_cancel_transaction,
            related_transaction_i_ds=related_transaction_i_ds,
            last_transaction_id=last_transaction_id,
        )
        cancel_order_response_200.additional_properties = d
        return cancel_order_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="UserInfoExternal")


@dataclasses.dataclass
class UserInfoExternal:
    """A representation of user information, as available to external (3rd party) clients.

    Attributes:
        user_id (Optional[int]): The user's OANDA-assigned user ID.
        country (Optional[str]): The country that the user is based in.
        fifo (Optional[bool]): Flag indicating if the the user's Accounts adhere to FIFO execution rules."""

    user_id: Optional[int]
    country: Optional[str]
    fifo: Optional[bool]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("userID", None)
        country = d.pop("country", None)
        fifo = d.pop("FIFO", None)
        user_info_external = cls(user_id=user_id, country=country, fifo=fifo)
        user_info_external.additional_properties = d
        return user_info_external

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

LimitOrderTransactionTriggerCondition = Literal[
    "ASK", "BID", "DEFAULT", "INVERSE", "MID"
]

LIMIT_ORDER_TRANSACTION_TRIGGER_CONDITION_VALUES: Set[
    LimitOrderTransactionTriggerCondition
] = {
    "ASK",
    "BID",
    "DEFAULT",
    "INVERSE",
    "MID",
}


def check_limit_order_transaction_trigger_condition(
    value: str,
) -> LimitOrderTransactionTriggerCondition:
    if value in LIMIT_ORDER_TRANSACTION_TRIGGER_CONDITION_VALUES:
        return cast(LimitOrderTransactionTriggerCondition, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_TRANSACTION_TRIGGER_CONDITION_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="OpenTradeFinancing")


@dataclasses.dataclass
class OpenTradeFinancing:
    """OpenTradeFinancing is used to pay/collect daily financing charge for an open Trade within an Account

    Attributes:
        trade_id (Optional[str]): The ID of the Trade that financing is being paid/collected for.
        financing (Optional[str]): The amount of financing paid/collected for the Trade."""

    trade_id: Optional[str]
    financing: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        trade_id = d.pop("tradeID", None)
        financing = d.pop("financing", None)
        open_trade_financing = cls(trade_id=trade_id, financing=financing)
        open_trade_financing.additional_properties = d
        return open_trade_financing

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .transfer_funds_transaction_funding_reason import (
    TransferFundsTransactionFundingReason,
)
from .transfer_funds_transaction_funding_reason import (
    check_transfer_funds_transaction_funding_reason,
)
from .transfer_funds_transaction_type import TransferFundsTransactionType
from .transfer_funds_transaction_type import check_transfer_funds_transaction_type
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TransferFundsTransaction")


@dataclasses.dataclass
class TransferFundsTransaction:
    """A TransferFundsTransaction represents the transfer of funds in/out of an Account.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[TransferFundsTransactionType]): The Type of the Transaction. Always set to "TRANSFER_FUNDS"
            in a TransferFundsTransaction.
        amount (Optional[str]): The amount to deposit/withdraw from the Account in the Account's home currency. A
            positive value indicates a deposit, a negative value indicates a withdrawal.
        funding_reason (Optional[TransferFundsTransactionFundingReason]): The reason that an Account is being
            funded.
        comment (Optional[str]): An optional comment that may be attached to a fund transfer for audit purposes
        account_balance (Optional[str]): The Account's balance after funds are transferred."""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[TransferFundsTransactionType]
    amount: Optional[str]
    funding_reason: Optional[TransferFundsTransactionFundingReason]
    comment: Optional[str]
    account_balance: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[TransferFundsTransactionType]
        if _type is None:
            type = None
        else:
            type = check_transfer_funds_transaction_type(_type)
        amount = d.pop("amount", None)
        _funding_reason = d.pop("fundingReason", None)
        funding_reason: Optional[TransferFundsTransactionFundingReason]
        if _funding_reason is None:
            funding_reason = None
        else:
            funding_reason = check_transfer_funds_transaction_funding_reason(
                _funding_reason
            )
        comment = d.pop("comment", None)
        account_balance = d.pop("accountBalance", None)
        transfer_funds_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            amount=amount,
            funding_reason=funding_reason,
            comment=comment,
            account_balance=account_balance,
        )
        transfer_funds_transaction.additional_properties = d
        return transfer_funds_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

TakeProfitOrderRejectTransactionTriggerCondition = Literal[
    "ASK", "BID", "DEFAULT", "INVERSE", "MID"
]
TAKE_PROFIT_ORDER_REJECT_TRANSACTION_TRIGGER_CONDITION_VALUES: Set[
    TakeProfitOrderRejectTransactionTriggerCondition
] = {"ASK", "BID", "DEFAULT", "INVERSE", "MID"}


def check_take_profit_order_reject_transaction_trigger_condition(
    value: str,
) -> TakeProfitOrderRejectTransactionTriggerCondition:
    if value in TAKE_PROFIT_ORDER_REJECT_TRANSACTION_TRIGGER_CONDITION_VALUES:
        return cast(TakeProfitOrderRejectTransactionTriggerCondition, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TAKE_PROFIT_ORDER_REJECT_TRANSACTION_TRIGGER_CONDITION_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="CalculatedTradeState")


@dataclasses.dataclass
class CalculatedTradeState:
    """The dynamic (calculated) state of an open Trade

    Attributes:
        id (Optional[str]): The Trade's ID.
        unrealized_pl (Optional[str]): The Trade's unrealized profit/loss.
        margin_used (Optional[str]): Margin currently used by the Trade."""

    id: Optional[str]
    unrealized_pl: Optional[str]
    margin_used: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        unrealized_pl = d.pop("unrealizedPL", None)
        margin_used = d.pop("marginUsed", None)
        calculated_trade_state = cls(
            id=id, unrealized_pl=unrealized_pl, margin_used=margin_used
        )
        calculated_trade_state.additional_properties = d
        return calculated_trade_state

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)
